﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing.Design;
using System.Windows.Forms.Design;
using System.Windows.Forms;
using Microsoft.VisualStudio.Modeling.Diagrams;

namespace DSLFactory.VSCTDesigner
{
    class GuidSymbolEditor : UITypeEditor
    {
        private IWindowsFormsEditorService _edService;

        public override object EditValue(System.ComponentModel.ITypeDescriptorContext context, IServiceProvider provider, object value)
        {
            NodeShape shape = context.Instance as NodeShape;
            Command cmd = shape.ModelElement as Command;

            IList<CommandTable> cts = cmd.Store.ElementDirectory.FindElements<CommandTable>();
            if( cts != null && cts.Count > 0 )
            {
                CommandTable ct = cts[0];
                _edService = (IWindowsFormsEditorService)provider.GetService(typeof(IWindowsFormsEditorService));

                ListBox list = new ListBox();
                list.Click += new EventHandler(list_Click);
                foreach( var id in ct.Symbols )
                {
                    list.Items.Add(id.Name);
                }
                try
                {
                    if(cmd is ExternCommand && ct.NativeCommandTable != null )
                    {
                        foreach( var str in ct.NativeCommandTable.GetGuidSymbols() )
                        {
                            list.Items.Add(str);
                        }
                    }
                }
                catch { }

                _edService.DropDownControl(list);
                if( list.SelectedIndex >= 0 )
                {
                    string val = (string)list.Items[list.SelectedIndex];
                    return val;
                }
            }
            return base.EditValue(context, provider, value);
        }

        void list_Click(object sender, EventArgs e)
        {
            _edService.CloseDropDown();
        }

        public override UITypeEditorEditStyle GetEditStyle(System.ComponentModel.ITypeDescriptorContext context)
        {
            return UITypeEditorEditStyle.DropDown;
        }
    }
}
